@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the 
  *    documentation and/or other materials provided with the distribution. 
  *
- * 3. All advertising materials mentioning features or use of this software 
- *    must display the following acknowledgement: 
- *      This product includes software developed by Kungliga Tekniska 
- *      Hgskolan and its contributors. 
- *
- * 4. Neither the name of the Institute nor the names of its contributors 
+ * 3. Neither the name of the Institute nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software 
  *    without specific prior written permission. 
  *
@@ -38,7 +33,8 @@
 
 #include "kadm5_locl.h"
 
-RCSID("$Id: init_s.c,v 1.7 1999/02/20 15:07:28 joda Exp $");
+RCSID("$Id: init_s.c,v 1.10 2000/12/31 08:01:16 assar Exp $");
+
 
 static kadm5_ret_t 
 kadm5_s_init_with_context(krb5_context context,
@@ -54,29 +50,23 @@ kadm5_s_init_with_context(krb5_context context,
     ret = _kadm5_s_init_context(&ctx, realm_params, context);
     if(ret)
 	return ret;
-    ret = hdb_create(ctx->context, &ctx->db, NULL);
+
+    assert(ctx->config.dbname != NULL);
+    assert(ctx->config.stash_file != NULL);
+    assert(ctx->config.acl_file != NULL);
+    assert(ctx->log_context.log_file != NULL);
+    assert(ctx->log_context.socket_name.sun_path[0] != '\0');
+
+    ret = hdb_create(ctx->context, &ctx->db, ctx->config.dbname);
     if(ret)
 	return ret;
     ret = hdb_set_master_keyfile (ctx->context, 
-				  ctx->db, NULL); /* XXX get from conf */
+				  ctx->db, ctx->config.stash_file);
     if(ret)
 	return ret;
 
-    ctx->acl_file = HDB_DB_DIR "/kadmind.acl";  /* XXX get from conf */
-    
-    ctx->log_context.log_file = HDB_DB_DIR "/log";
-
     ctx->log_context.log_fd   = -1;
 
-    {
-	struct sockaddr_un *un = &ctx->log_context.socket_name;
-
-	memset(un, 0, sizeof(*un));
-	un->sun_family = AF_UNIX;
-	strncpy (un->sun_path, KADM5_LOG_SIGNAL, sizeof(un->sun_path));
-	un->sun_path[sizeof(un->sun_path) - 1] = '\0';
-    }
-
     ctx->log_context.socket_fd = socket (AF_UNIX, SOCK_DGRAM, 0);
 
     ret = krb5_parse_name(ctx->context, client_name, &ctx->caller);
@@ -123,7 +113,9 @@ kadm5_s_init_with_password(const char *client_name,
     kadm5_ret_t ret;
     kadm5_server_context *ctx;
 
-    krb5_init_context(&context);
+    ret = krb5_init_context(&context);
+    if (ret)
+	return ret;
     ret = kadm5_s_init_with_password_ctx(context, 
 					 client_name, 
 					 password, 
@@ -173,7 +165,9 @@ kadm5_s_init_with_skey(const char *client_name,
     kadm5_ret_t ret;
     kadm5_server_context *ctx;
 
-    krb5_init_context(&context);
+    ret = krb5_init_context(&context);
+    if (ret)
+	return ret;
     ret = kadm5_s_init_with_skey_ctx(context, 
 				     client_name, 
 				     keytab, 
@@ -223,7 +217,9 @@ kadm5_s_init_with_creds(const char *client_name,
     kadm5_ret_t ret;
     kadm5_server_context *ctx;
 
-    krb5_init_context(&context);
+    ret = krb5_init_context(&context);
+    if (ret)
+	return ret;
     ret = kadm5_s_init_with_creds_ctx(context, 
 				      client_name, 
 				      ccache, 
