@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997 - 2003 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the 
  *    documentation and/or other materials provided with the distribution. 
  *
- * 3. All advertising materials mentioning features or use of this software 
- *    must display the following acknowledgement: 
- *      This product includes software developed by Kungliga Tekniska 
- *      Hgskolan and its contributors. 
- *
- * 4. Neither the name of the Institute nor the names of its contributors 
+ * 3. Neither the name of the Institute nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software 
  *    without specific prior written permission. 
  *
@@ -38,7 +33,7 @@
 
 #include "kadm5_locl.h"
 
-RCSID("$Id: log.c,v 1.7 1998/03/01 02:38:19 assar Exp $");
+RCSID("$Id: log.c,v 1.20 2003/04/16 17:56:55 lha Exp $");
 
 /*
  * A log record consists of:
@@ -54,8 +49,8 @@ RCSID("$Id: log.c,v 1.7 1998/03/01 02:38:19 assar Exp $");
  */
 
 kadm5_ret_t
-kadm5_log_get_version (int fd,
-		       u_int32_t *ver)
+kadm5_log_get_version_fd (int fd,
+			  u_int32_t *ver)
 {
     int ret;
     krb5_storage *sp;
@@ -69,7 +64,7 @@ kadm5_log_get_version (int fd,
 	return 0;
     }
     sp = krb5_storage_from_fd (fd);
-    sp->seek(sp, -4, SEEK_CUR);
+    krb5_storage_seek(sp, -4, SEEK_CUR);
     krb5_ret_int32 (sp, &old_version);
     *ver = old_version;
     krb5_storage_free(sp);
@@ -77,6 +72,21 @@ kadm5_log_get_version (int fd,
     return 0;
 }
 
+kadm5_ret_t
+kadm5_log_get_version (kadm5_server_context *context, u_int32_t *ver)
+{
+    return kadm5_log_get_version_fd (context->log_context.log_fd, ver);
+}
+
+kadm5_ret_t
+kadm5_log_set_version (kadm5_server_context *context, u_int32_t vno)
+{
+    kadm5_log_context *log_context = &context->log_context;
+
+    log_context->version = vno;
+    return 0;
+}
+
 kadm5_ret_t
 kadm5_log_init (kadm5_server_context *context)
 {
@@ -94,7 +104,7 @@ kadm5_log_init (kadm5_server_context *context)
 	return errno;
     }
 
-    ret = kadm5_log_get_version (fd, &log_context->version);
+    ret = kadm5_log_get_version_fd (fd, &log_context->version);
     if (ret)
 	return ret;
 
@@ -102,6 +112,30 @@ kadm5_log_init (kadm5_server_context *context)
     return 0;
 }
 
+kadm5_ret_t
+kadm5_log_reinit (kadm5_server_context *context)
+{
+    int fd;
+    kadm5_log_context *log_context = &context->log_context;
+
+    if (log_context->log_fd != -1) {
+	close (log_context->log_fd);
+	log_context->log_fd = -1;
+    }
+    fd = open (log_context->log_file, O_RDWR | O_CREAT | O_TRUNC, 0600);
+    if (fd < 0)
+	return errno;
+    if (flock (fd, LOCK_EX) < 0) {
+	close (fd);
+	return errno;
+    }
+
+    log_context->version = 0;
+    log_context->log_fd  = fd;
+    return 0;
+}
+
+
 kadm5_ret_t
 kadm5_log_end (kadm5_server_context *context)
 {
@@ -167,7 +201,7 @@ kadm5_log_flush (kadm5_log_context *log_context,
      * Try to send a signal to any running `ipropd-master'
      */
     sendto (log_context->socket_fd,
-	    &log_context->version,
+	    (void *)&log_context->version,
 	    sizeof(log_context->version),
 	    0,
 	    (struct sockaddr *)&log_context->socket_name,
@@ -203,7 +237,7 @@ kadm5_log_create (kadm5_server_context *context,
 	return ret;
     }
     krb5_store_int32 (sp, value.length);
-    sp->store(sp, value.data, value.length);
+    krb5_storage_write(sp, value.data, value.length);
     krb5_store_int32 (sp, value.length);
     krb5_data_free (&value);
     ret = kadm5_log_postamble (log_context, sp);
@@ -234,8 +268,10 @@ kadm5_log_replay_create (kadm5_server_context *context,
     krb5_data data;
     hdb_entry ent;
 
-    krb5_data_alloc (&data, len);
-    sp->fetch (sp, data.data, len);
+    ret = krb5_data_alloc (&data, len);
+    if (ret)
+	return ret;
+    krb5_storage_read (sp, data.data, len);
     ret = hdb_value2entry (context->context, &data, &ent);
     krb5_data_free(&data);
     if (ret)
@@ -266,12 +302,12 @@ kadm5_log_delete (kadm5_server_context *context,
 	return ret;
     }
     krb5_store_int32 (sp, 0);
-    off = sp->seek (sp, 0, SEEK_CUR);
+    off = krb5_storage_seek (sp, 0, SEEK_CUR);
     krb5_store_principal (sp, princ);
-    len = sp->seek (sp, 0, SEEK_CUR) - off;
-    sp->seek(sp, -(len + 4), SEEK_CUR);
+    len = krb5_storage_seek (sp, 0, SEEK_CUR) - off;
+    krb5_storage_seek(sp, -(len + 4), SEEK_CUR);
     krb5_store_int32 (sp, len);
-    sp->seek(sp, len, SEEK_CUR);
+    krb5_storage_seek(sp, len, SEEK_CUR);
     krb5_store_int32 (sp, len);
     if (ret) {
 	krb5_storage_free (sp);
@@ -305,7 +341,7 @@ kadm5_log_replay_delete (kadm5_server_context *context,
 
     krb5_ret_principal (sp, &ent.principal);
 
-    ret = context->db->delete(context->context, context->db, &ent);
+    ret = context->db->remove(context->context, context->db, &ent);
     krb5_free_principal (context->context, ent.principal);
     return ret;
 }
@@ -339,15 +375,15 @@ kadm5_log_rename (kadm5_server_context *context,
 	return ret;
     }
     krb5_store_int32 (sp, 0);
-    off = sp->seek (sp, 0, SEEK_CUR);
+    off = krb5_storage_seek (sp, 0, SEEK_CUR);
     krb5_store_principal (sp, source);
-    sp->store(sp, value.data, value.length);
+    krb5_storage_write(sp, value.data, value.length);
     krb5_data_free (&value);
-    len = sp->seek (sp, 0, SEEK_CUR) - off;
+    len = krb5_storage_seek (sp, 0, SEEK_CUR) - off;
 
-    sp->seek(sp, -(len + 4), SEEK_CUR);
+    krb5_storage_seek(sp, -(len + 4), SEEK_CUR);
     krb5_store_int32 (sp, len);
-    sp->seek(sp, len, SEEK_CUR);
+    krb5_storage_seek(sp, len, SEEK_CUR);
     krb5_store_int32 (sp, len);
     if (ret) {
 	krb5_storage_free (sp);
@@ -383,12 +419,16 @@ kadm5_log_replay_rename (kadm5_server_context *context,
     off_t off;
     size_t princ_len, data_len;
 
-    off = sp->seek(sp, 0, SEEK_CUR);
+    off = krb5_storage_seek(sp, 0, SEEK_CUR);
     krb5_ret_principal (sp, &source);
-    princ_len = sp->seek(sp, 0, SEEK_CUR) - off;
+    princ_len = krb5_storage_seek(sp, 0, SEEK_CUR) - off;
     data_len = len - princ_len;
-    krb5_data_alloc (&value, data_len);
-    sp->fetch (sp, value.data, data_len);
+    ret = krb5_data_alloc (&value, data_len);
+    if (ret) {
+	krb5_free_principal (context->context, source);
+	return ret;
+    }
+    krb5_storage_read (sp, value.data, data_len);
     ret = hdb_value2entry (context->context, &value, &target_ent);
     krb5_data_free(&value);
     if (ret) {
@@ -402,7 +442,7 @@ kadm5_log_replay_rename (kadm5_server_context *context,
 	return ret;
     }
     source_ent.principal = source;
-    ret = context->db->delete (context->context, context->db, &source_ent);
+    ret = context->db->remove (context->context, context->db, &source_ent);
     krb5_free_principal (context->context, source);
     return ret;
 }
@@ -438,7 +478,7 @@ kadm5_log_modify (kadm5_server_context *context,
     len = value.length + 4;
     krb5_store_int32 (sp, len);
     krb5_store_int32 (sp, mask);
-    sp->store(sp, value.data, value.length);
+    krb5_storage_write (sp, value.data, value.length);
     krb5_data_free (&value);
     krb5_store_int32 (sp, len);
     if (ret) {
@@ -475,26 +515,37 @@ kadm5_log_replay_modify (kadm5_server_context *context,
 
     krb5_ret_int32 (sp, &mask);
     len -= 4;
-    krb5_data_alloc (&value, len);
-    sp->fetch (sp, value.data, len);
+    ret = krb5_data_alloc (&value, len);
+    if (ret)
+	return ret;
+    krb5_storage_read (sp, value.data, len);
     ret = hdb_value2entry (context->context, &value, &log_ent);
     krb5_data_free(&value);
     if (ret)
 	return ret;
     ent.principal = log_ent.principal;
     log_ent.principal = NULL;
-    ret = context->db->fetch(context->context, context->db, &ent);
+    ret = context->db->fetch(context->context, context->db, 
+			     HDB_F_DECRYPT, &ent);
     if (ret)
 	return ret;
     if (mask & KADM5_PRINC_EXPIRE_TIME) {
-	if (ent.valid_end == NULL)
-	    ent.valid_end = malloc(sizeof(*ent.valid_end));
-	*ent.valid_end = *log_ent.valid_end;
+	if (log_ent.valid_end == NULL) {
+	    ent.valid_end = NULL;
+	} else {
+	    if (ent.valid_end == NULL)
+		ent.valid_end = malloc(sizeof(*ent.valid_end));
+	    *ent.valid_end = *log_ent.valid_end;
+	}
     }
     if (mask & KADM5_PW_EXPIRATION) {
-	if (ent.pw_end == NULL)
-	    ent.pw_end = malloc(sizeof(*ent.pw_end));
-	*ent.pw_end = *log_ent.pw_end;
+	if (log_ent.pw_end == NULL) {
+	    ent.pw_end = NULL;
+	} else {
+	    if (ent.pw_end == NULL)
+		ent.pw_end = malloc(sizeof(*ent.pw_end));
+	    *ent.pw_end = *log_ent.pw_end;
+	}
     }
     if (mask & KADM5_LAST_PWD_CHANGE) {
 	abort ();		/* XXX */
@@ -503,9 +554,13 @@ kadm5_log_replay_modify (kadm5_server_context *context,
 	ent.flags = log_ent.flags;
     }
     if (mask & KADM5_MAX_LIFE) {
-	if (ent.max_life == NULL)
-	    ent.max_life = malloc (sizeof(*ent.max_life));
-	*ent.max_life = *log_ent.max_life;
+	if (log_ent.max_life == NULL) {
+	    ent.max_life = NULL;
+	} else {
+	    if (ent.max_life == NULL)
+		ent.max_life = malloc (sizeof(*ent.max_life));
+	    *ent.max_life = *log_ent.max_life;
+	}
     }
     if ((mask & KADM5_MOD_TIME) && (mask & KADM5_MOD_NAME)) {
 	if (ent.modified_by == NULL) {
@@ -530,9 +585,13 @@ kadm5_log_replay_modify (kadm5_server_context *context,
 	abort ();		/* XXX */
     }
     if (mask & KADM5_MAX_RLIFE) {
-	if (ent.max_renew == NULL)
-	    ent.max_renew = malloc (sizeof(*ent.max_renew));
-	*ent.max_renew = *log_ent.max_renew;
+	if (log_ent.max_renew == NULL) {
+	    ent.max_renew = NULL;
+	} else {
+	    if (ent.max_renew == NULL)
+		ent.max_renew = malloc (sizeof(*ent.max_renew));
+	    *ent.max_renew = *log_ent.max_renew;
+	}
     }
     if (mask & KADM5_LAST_SUCCESS) {
 	abort ();		/* XXX */
@@ -559,19 +618,66 @@ kadm5_log_replay_modify (kadm5_server_context *context,
 	    copy_Key(&log_ent.keys.val[i],
 		     &ent.keys.val[i]);
     }
-    ret = context->db->store(context->context, context->db, 1, &ent);
+    ret = context->db->store(context->context, context->db, 
+			     HDB_F_REPLACE, &ent);
     hdb_free_entry (context->context, &ent);
     hdb_free_entry (context->context, &log_ent);
     return ret;
 }
 
+/*
+ * Add a `nop' operation to the log.
+ */
+
+kadm5_ret_t
+kadm5_log_nop (kadm5_server_context *context)
+{
+    krb5_storage *sp;
+    kadm5_ret_t ret;
+    kadm5_log_context *log_context = &context->log_context;
+
+    sp = krb5_storage_emem();
+    ret = kadm5_log_preamble (context, sp, kadm_nop);
+    if (ret) {
+	krb5_storage_free (sp);
+	return ret;
+    }
+    krb5_store_int32 (sp, 0);
+    krb5_store_int32 (sp, 0);
+    ret = kadm5_log_postamble (log_context, sp);
+    if (ret) {
+	krb5_storage_free (sp);
+	return ret;
+    }
+    ret = kadm5_log_flush (log_context, sp);
+    krb5_storage_free (sp);
+    if (ret)
+	return ret;
+    ret = kadm5_log_end (context);
+    return ret;
+}
+
+/*
+ * Read a `nop' log operation from `sp' and apply it.
+ */
+
+kadm5_ret_t
+kadm5_log_replay_nop (kadm5_server_context *context,
+		      u_int32_t ver,
+		      u_int32_t len,
+		      krb5_storage *sp)
+{
+    return 0;
+}
+
 /*
  * Call `func' for each log record in the log in `context'
  */
 
 kadm5_ret_t
 kadm5_log_foreach (kadm5_server_context *context,
-		   void (*func)(u_int32_t ver,
+		   void (*func)(kadm5_server_context *server_context,
+				u_int32_t ver,
 				time_t timestamp,
 				enum kadm_ops op,
 				u_int32_t len,
@@ -590,8 +696,8 @@ kadm5_log_foreach (kadm5_server_context *context,
 	krb5_ret_int32 (sp, &timestamp);
 	krb5_ret_int32 (sp, &op);
 	krb5_ret_int32 (sp, &len);
-	(*func)(ver, timestamp, op, len, sp);
-	sp->seek(sp, 8, SEEK_CUR);
+	(*func)(context, ver, timestamp, op, len, sp);
+	krb5_storage_seek(sp, 8, SEEK_CUR);
     }
     return 0;
 }
@@ -606,7 +712,7 @@ kadm5_log_goto_end (int fd)
     krb5_storage *sp;
 
     sp = krb5_storage_from_fd (fd);
-    sp->seek(sp, 0, SEEK_END);
+    krb5_storage_seek(sp, 0, SEEK_END);
     return sp;
 }
 
@@ -624,13 +730,13 @@ kadm5_log_previous (krb5_storage *sp,
     off_t off;
     int32_t tmp;
 
-    sp->seek(sp, -8, SEEK_CUR);
+    krb5_storage_seek(sp, -8, SEEK_CUR);
     krb5_ret_int32 (sp, &tmp);
     *len = tmp;
     krb5_ret_int32 (sp, &tmp);
     *ver = tmp;
     off = 24 + *len;
-    sp->seek(sp, -off, SEEK_CUR);
+    krb5_storage_seek(sp, -off, SEEK_CUR);
     krb5_ret_int32 (sp, &tmp);
     assert(tmp == *ver);
     krb5_ret_int32 (sp, &tmp);
@@ -662,7 +768,46 @@ kadm5_log_replay (kadm5_server_context *context,
 	return kadm5_log_replay_rename (context, ver, len, sp);
     case kadm_modify :
 	return kadm5_log_replay_modify (context, ver, len, sp);
+    case kadm_nop :
+	return kadm5_log_replay_nop (context, ver, len, sp);
     default :
 	return KADM5_FAILURE;
     }
 }
+
+/*
+ * truncate the log - i.e. create an empty file with just (nop vno + 2)
+ */
+
+kadm5_ret_t
+kadm5_log_truncate (kadm5_server_context *server_context)
+{
+    kadm5_ret_t ret;
+    u_int32_t vno;
+
+    ret = kadm5_log_init (server_context);
+    if (ret)
+	return ret;
+
+    ret = kadm5_log_get_version (server_context, &vno);
+    if (ret)
+	return ret;
+
+    ret = kadm5_log_reinit (server_context);
+    if (ret)
+	return ret;
+
+    ret = kadm5_log_set_version (server_context, vno + 1);
+    if (ret)
+	return ret;
+
+    ret = kadm5_log_nop (server_context);
+    if (ret)
+	return ret;
+
+    ret = kadm5_log_end (server_context);
+    if (ret)
+	return ret;
+    return 0;
+
+}
