@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the 
  *    documentation and/or other materials provided with the distribution. 
  *
- * 3. All advertising materials mentioning features or use of this software 
- *    must display the following acknowledgement: 
- *      This product includes software developed by Kungliga Tekniska 
- *      Hgskolan and its contributors. 
- *
- * 4. Neither the name of the Institute nor the names of its contributors 
+ * 3. Neither the name of the Institute nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software 
  *    without specific prior written permission. 
  *
@@ -38,71 +33,84 @@
 
 #include <krb5_locl.h>
 
-RCSID("$Id: mk_req.c,v 1.16 1999/10/10 01:23:04 assar Exp $");
+RCSID("$Id: mk_req.c,v 1.24 2001/06/18 20:05:52 joda Exp $");
+
+krb5_error_code
+krb5_mk_req_exact(krb5_context context,
+		  krb5_auth_context *auth_context,
+		  const krb5_flags ap_req_options,
+		  const krb5_principal server,
+		  krb5_data *in_data,
+		  krb5_ccache ccache,
+		  krb5_data *outbuf)
+{
+    krb5_error_code ret;
+    krb5_creds this_cred, *cred;
+
+    memset(&this_cred, 0, sizeof(this_cred));
+
+    ret = krb5_cc_get_principal(context, ccache, &this_cred.client);
+  
+    if(ret)
+	return ret;
+
+    ret = krb5_copy_principal (context, server, &this_cred.server);
+    if (ret) {
+	krb5_free_creds_contents (context, &this_cred);
+	return ret;
+    }
+
+    this_cred.times.endtime = 0;
+    if (auth_context && *auth_context && (*auth_context)->keytype)
+	this_cred.session.keytype = (*auth_context)->keytype;
+
+    ret = krb5_get_credentials (context, 0, ccache, &this_cred, &cred);
+    krb5_free_creds_contents(context, &this_cred);
+    if (ret)
+	return ret;
+
+    ret = krb5_mk_req_extended (context,
+				auth_context,
+				ap_req_options,
+				in_data,
+				cred,
+				outbuf);
+    krb5_free_creds(context, cred);
+    return ret;
+}
 
 krb5_error_code
 krb5_mk_req(krb5_context context,
 	    krb5_auth_context *auth_context,
 	    const krb5_flags ap_req_options,
-	    char *service,
-	    char *hostname,
+	    const char *service,
+	    const char *hostname,
 	    krb5_data *in_data,
 	    krb5_ccache ccache,
 	    krb5_data *outbuf)
 {
-    krb5_error_code r;
-    krb5_creds this_cred, *cred;
+    krb5_error_code ret;
     char **realms;
-    krb5_data realm_data;
-    char **p;
     char *real_hostname;
+    krb5_principal server;
 
-    memset(&this_cred, 0, sizeof(this_cred));
+    ret = krb5_expand_hostname_realms (context, hostname,
+				       &real_hostname, &realms);
+    if (ret)
+	return ret;
 
-    r = krb5_cc_get_principal(context, ccache, &this_cred.client);
-  
-    if(r)
-	return r;
-
-    r = krb5_expand_hostname (context, hostname, &real_hostname);
-    if (r) {
-	krb5_free_principal (context, this_cred.client);
-	return r;
-    }
-
-    r = krb5_get_host_realm(context, real_hostname, &realms);
-    if (r) {
-	krb5_free_principal (context, this_cred.client);
-	return r;
-    }
-    realm_data.length = strlen(*realms);
-    realm_data.data   = *realms;
-
-    r = krb5_build_principal (context, &this_cred.server,
-			      strlen(*realms),
-			      *realms,
-			      service,
-			      real_hostname,
-			      NULL);
+    ret = krb5_build_principal (context, &server,
+				strlen(*realms),
+				*realms,
+				service,
+				real_hostname,
+				NULL);
     free (real_hostname);
     krb5_free_host_realm (context, realms);
-
-    if (r) {
-	krb5_free_principal (context, this_cred.client);
-	return r;
-    }
-    this_cred.times.endtime = 0;
-    if (auth_context && *auth_context && (*auth_context)->keytype)
-	this_cred.session.keytype = (*auth_context)->keytype;
-
-    r = krb5_get_credentials (context, 0, ccache, &this_cred, &cred);
-    if (r)
-	return r;
-
-    return krb5_mk_req_extended (context,
-				 auth_context,
-				 ap_req_options,
-				 in_data,
-				 cred,
-				 outbuf);
+    if (ret)
+	return ret;
+    ret = krb5_mk_req_exact (context, auth_context, ap_req_options,
+			     server, in_data, ccache, outbuf);
+    krb5_free_principal (context, server);
+    return ret;
 }
